@@ -37,6 +37,8 @@
 #include <libnautilus-private/nautilus-icon-factory.h>
 #include <string.h>
 
+typedef gboolean (* FilePredicate) (NautilusFile *);
+
 /* The user_data of the GtkTreeIter is the TreeNode pointer.
  * It's NULL for the dummy node. If it's NULL, then user_data2
  * is the TreeNode pointer to the parent.
@@ -71,6 +73,7 @@ struct TreeNode {
 
 	TreeNode *first_child;
 	gboolean done_loading;
+	gboolean inserting_first_child;
 };
 
 struct NautilusTreeModelDetails {
@@ -78,10 +81,13 @@ struct NautilusTreeModelDetails {
 	GHashTable *file_to_node_map;
 	
 	TreeNode *root_node;	
-	guint root_node_changed_id;
 	gboolean root_node_parented;
 
-	guint destroy_unneeded_children_idle_id;
+	guint monitoring_update_idle_id;
+
+	gboolean show_hidden_files;
+	gboolean show_backup_files;
+	gboolean show_only_directories;
 };
 
 typedef struct {
@@ -91,9 +97,9 @@ typedef struct {
 
 static GObjectClass *parent_class;
 
-static void schedule_destroy_unneeded_children (NautilusTreeModel *model);
-static void destroy_node_without_reporting     (NautilusTreeModel *model,
-						TreeNode          *node);
+static void schedule_monitoring_update     (NautilusTreeModel *model);
+static void destroy_node_without_reporting (NautilusTreeModel *model,
+					    TreeNode          *node);
 
 static void
 object_unref_if_not_NULL (gpointer object)
@@ -277,7 +283,9 @@ static gboolean
 tree_node_has_dummy_child (TreeNode *node)
 {
 	return node->directory != NULL
-		&& (!node->done_loading || node->first_child == NULL);
+		&& (!node->done_loading
+		    || node->first_child == NULL
+		    || node->inserting_first_child);
 }
 
 static int
@@ -393,7 +401,7 @@ decrement_ref_count (NautilusTreeModel *model, TreeNode *node, int count)
 {
 	node->all_children_ref_count -= count;
 	if (node->all_children_ref_count == 0) {
-		schedule_destroy_unneeded_children (model);
+		schedule_monitoring_update (model);
 	}
 }
 
@@ -559,8 +567,6 @@ stop_monitoring_directory (NautilusTreeModel *model, TreeNode *node)
 {
 	NautilusDirectory *directory;
 
-	node->done_loading = FALSE;
-
 	if (node->done_loading_id == 0) {
 		g_assert (node->files_added_id == 0);
 		g_assert (node->files_changed_id == 0);
@@ -633,6 +639,27 @@ destroy_children (NautilusTreeModel *model, TreeNode *parent)
 	}
 }
 
+static void
+destroy_children_by_function (NautilusTreeModel *model, TreeNode *parent, FilePredicate f)
+{
+	TreeNode *child, *next;
+
+	for (child = parent->first_child; child != NULL; child = next) {
+		next = child->next;
+		if (f (child->file)) {
+			destroy_node (model, child);
+		} else {
+			destroy_children_by_function (model, child, f);
+		}
+	}
+}
+
+static void
+destroy_by_function (NautilusTreeModel *model, FilePredicate f)
+{
+	destroy_children_by_function (model, model->details->root_node, f);
+}
+
 static gboolean
 update_node_without_reporting (NautilusTreeModel *model, TreeNode *node)
 {
@@ -659,9 +686,24 @@ update_node_without_reporting (NautilusTreeModel *model, TreeNode *node)
 static void
 insert_node (NautilusTreeModel *model, TreeNode *parent, TreeNode *node)
 {
+	gboolean parent_empty;
+
+	parent_empty = parent->first_child == NULL;
+	if (parent_empty) {
+		parent->inserting_first_child = TRUE;
+	}
+
 	tree_node_parent (node, parent);
+
 	update_node_without_reporting (model, node);
 	report_node_inserted (model, node);
+
+	if (parent_empty) {
+		parent->inserting_first_child = FALSE;
+		if (!tree_node_has_dummy_child (parent)) {
+			report_dummy_row_deleted (model, parent);
+		}
+	}
 }
 
 static void
@@ -687,6 +729,33 @@ reparent_node (NautilusTreeModel *model, TreeNode *node)
 	insert_node (model, new_parent, node);
 }
 
+static gboolean
+should_show_file (NautilusTreeModel *model, NautilusFile *file)
+{
+	gboolean should;
+
+	should = nautilus_file_should_show (file,
+					    model->details->show_hidden_files,
+					    model->details->show_backup_files);
+
+	if (should
+	    && model->details->show_only_directories
+	    &&! nautilus_file_is_directory (file)) {
+		should = FALSE;
+	}
+
+	if (should && nautilus_file_is_gone (file)) {
+		should = FALSE;
+	}
+
+	if (!should && model->details->root_node != NULL
+	    && file == model->details->root_node->file) {
+		should = TRUE;
+	}
+
+	return should;
+}
+
 static void
 update_node (NautilusTreeModel *model, TreeNode *node)
 {
@@ -694,7 +763,7 @@ update_node (NautilusTreeModel *model, TreeNode *node)
 	gboolean had_directory, has_directory;
 	gboolean changed;
 
-	if (nautilus_file_is_gone (node->file)) {
+	if (!should_show_file (model, node->file)) {
 		destroy_node (model, node);
 		return;
 	}
@@ -741,7 +810,7 @@ process_file_change (NautilusTreeModel *model,
 		return;
 	}
 
-	if (nautilus_file_is_gone (file)) {
+	if (!should_show_file (model, file)) {
 		return;
 	}
 
@@ -768,29 +837,43 @@ files_changed_callback (NautilusDirectory *directory,
 	}
 }
 
+static void
+set_done_loading (NautilusTreeModel *model, TreeNode *node, gboolean done_loading)
+{
+	gboolean had_dummy;
+
+	if (node == NULL || node->done_loading == done_loading) {
+		return;
+	}
+
+	had_dummy = tree_node_has_dummy_child (node);
+
+	node->done_loading = done_loading;
+
+	if (tree_node_has_dummy_child (node)) {
+		if (had_dummy) {
+			report_dummy_row_contents_changed (model, node);
+		} else {
+			report_dummy_row_inserted (model, node);
+		}
+	} else {
+		if (had_dummy) {
+			report_dummy_row_deleted (model, node);
+		} else {
+			g_assert_not_reached ();
+		}
+	}
+}
+
 static void
 done_loading_callback (NautilusDirectory *directory,
 		       NautilusTreeModel *model)
 {
 	NautilusFile *file;
-	TreeNode *node;
 
 	file = nautilus_directory_get_corresponding_file (directory);
-	node = get_node_from_file (model, file);
+	set_done_loading (model, get_node_from_file (model, file), TRUE);
 	nautilus_file_unref (file);
-	if (node == NULL || node->done_loading) {
-		return;
-	}
-
-	g_assert (tree_node_has_dummy_child (node));
-
-	node->done_loading = TRUE;
-
-	if (tree_node_has_dummy_child (node)) {
-		report_dummy_row_contents_changed (model, node);
-	} else {
-		report_dummy_row_deleted (model, node);
-	}
 }
 
 static GList *
@@ -804,28 +887,11 @@ get_tree_monitor_attributes (void)
 	return attrs;
 }
 
-static gboolean
-done_loading_idle_callback (gpointer callback_data)
-{
-	DoneLoadingParameters *p;
-
-	p = callback_data;
-	if (p->directory != NULL && p->model != NULL) {
-		done_loading_callback (p->directory, p->model);
-	}
-	eel_remove_weak_pointer (&p->model);
-	eel_remove_weak_pointer (&p->directory);
-	g_free (p);
-
-	return FALSE;
-}
-
 static void
 start_monitoring_directory (NautilusTreeModel *model, TreeNode *node)
 {
 	NautilusDirectory *directory;
 	GList *attrs;
-	DoneLoadingParameters *p;
 
 	if (node->done_loading_id != 0) {
 		return;
@@ -839,31 +905,21 @@ start_monitoring_directory (NautilusTreeModel *model, TreeNode *node)
 	node->done_loading_id = g_signal_connect
 		(directory, "done_loading",
 		 G_CALLBACK (done_loading_callback), model);
-	
 	node->files_added_id = g_signal_connect 
 		(directory, "files_added",
 		 G_CALLBACK (files_changed_callback), model);
-	
 	node->files_changed_id = g_signal_connect 
 		(directory, "files_changed",
 		 G_CALLBACK (files_changed_callback), model);	
+
+	set_done_loading (model, node, nautilus_directory_are_all_files_seen (directory));
 	
 	attrs = get_tree_monitor_attributes ();
-	nautilus_directory_file_monitor_add (directory, model, TRUE, TRUE, attrs,
-					     files_changed_callback, model);
+	nautilus_directory_file_monitor_add (directory, model,
+					     model->details->show_hidden_files,
+					     model->details->show_backup_files,
+					     attrs, files_changed_callback, model);
 	g_list_free (attrs);
-
-	if (nautilus_directory_are_all_files_seen (directory)) {
-		/* Can't just remove the dummy node in here, so we do
-		 * it at idle time.
-		 */
-		p = g_new (DoneLoadingParameters, 1);
-		p->directory = directory;
-		p->model = model;
-		eel_add_weak_pointer (&p->directory);
-		eel_add_weak_pointer (&p->model);
-		g_idle_add (done_loading_idle_callback, p);
-	}
 }
 
 static int
@@ -994,7 +1050,6 @@ static void
 nautilus_tree_model_get_value (GtkTreeModel *model, GtkTreeIter *iter, int column, GValue *value)
 {
 	TreeNode *node, *parent;
-	const char *dummy_node_str;
 
 	g_return_if_fail (NAUTILUS_IS_TREE_MODEL (model));
 	g_return_if_fail (iter_is_valid (NAUTILUS_TREE_MODEL (model), iter));
@@ -1006,8 +1061,8 @@ nautilus_tree_model_get_value (GtkTreeModel *model, GtkTreeIter *iter, int colum
 		g_value_init (value, G_TYPE_STRING);
 		if (node == NULL) {
 			parent = iter->user_data2;
-			dummy_node_str = parent->done_loading ? _("(Empty)") : _("Loading...");
-			g_value_set_static_string (value, dummy_node_str);
+			g_value_set_static_string (value, parent->done_loading
+						   ? _("(Empty)") : _("Loading..."));
 		} else {
 			g_value_set_string (value, tree_node_get_display_name (node));
 		}
@@ -1023,9 +1078,8 @@ nautilus_tree_model_get_value (GtkTreeModel *model, GtkTreeIter *iter, int colum
 	case NAUTILUS_TREE_MODEL_FONT_STYLE_COLUMN:
 		g_value_init (value, PANGO_TYPE_STYLE);
 		if (node == NULL) {
-			g_value_set_enum (value, PANGO_STYLE_OBLIQUE);
-		}
-		else {
+			g_value_set_enum (value, PANGO_STYLE_ITALIC);
+		} else {
 			g_value_set_enum (value, PANGO_STYLE_NORMAL);
 		}
 		break;
@@ -1134,14 +1188,16 @@ nautilus_tree_model_iter_n_children (GtkTreeModel *model, GtkTreeIter *iter)
 }
 
 static gboolean
-nautilus_tree_model_iter_nth_child (GtkTreeModel *model, GtkTreeIter *iter, GtkTreeIter *parent_iter, int n)
+nautilus_tree_model_iter_nth_child (GtkTreeModel *model, GtkTreeIter *iter,
+				    GtkTreeIter *parent_iter, int n)
 {
 	NautilusTreeModel *tree_model;
 	TreeNode *parent, *node;
 	int i;
 	
 	g_return_val_if_fail (NAUTILUS_IS_TREE_MODEL (model), FALSE);
-	g_return_val_if_fail (parent_iter == NULL || iter_is_valid (NAUTILUS_TREE_MODEL (model), parent_iter), FALSE);
+	g_return_val_if_fail (parent_iter == NULL
+			      || iter_is_valid (NAUTILUS_TREE_MODEL (model), parent_iter), FALSE);
 	
 	tree_model = NAUTILUS_TREE_MODEL (model);
 
@@ -1150,7 +1206,7 @@ nautilus_tree_model_iter_nth_child (GtkTreeModel *model, GtkTreeIter *iter, GtkT
 			return make_iter_invalid (iter);
 		}
 		return make_iter_for_node (tree_model->details->root_node,
-						  iter, tree_model->details->stamp);
+					   iter, tree_model->details->stamp);
 	}
 
 	parent = parent_iter->user_data;
@@ -1172,7 +1228,7 @@ nautilus_tree_model_iter_nth_child (GtkTreeModel *model, GtkTreeIter *iter, GtkT
 }
 
 static void
-destroy_unneeded_children (NautilusTreeModel *model, TreeNode *node)
+update_monitoring (NautilusTreeModel *model, TreeNode *node)
 {
 	TreeNode *child;
 
@@ -1181,47 +1237,47 @@ destroy_unneeded_children (NautilusTreeModel *model, TreeNode *node)
 		destroy_children (model, node);
 	} else {
 		for (child = node->first_child; child != NULL; child = child->next) {
-			destroy_unneeded_children (model, child);
+			update_monitoring (model, child);
 		}
+		start_monitoring_directory (model, node);
 	}
 }
 
 static gboolean
-destroy_unneeded_children_idle_callback (gpointer callback_data)
+update_monitoring_idle_callback (gpointer callback_data)
 {
 	NautilusTreeModel *model;
 
 	model = NAUTILUS_TREE_MODEL (callback_data);
-	model->details->destroy_unneeded_children_idle_id = 0;
-	destroy_unneeded_children (model, model->details->root_node);
+	model->details->monitoring_update_idle_id = 0;
+	update_monitoring (model, model->details->root_node);
 	return FALSE;
 }
 
 static void
-schedule_destroy_unneeded_children (NautilusTreeModel *model)
+schedule_monitoring_update (NautilusTreeModel *model)
 {
-	if (model->details->destroy_unneeded_children_idle_id == 0) {
-		model->details->destroy_unneeded_children_idle_id =
-			g_idle_add (destroy_unneeded_children_idle_callback, model);
+	if (model->details->monitoring_update_idle_id == 0) {
+		model->details->monitoring_update_idle_id =
+			g_idle_add (update_monitoring_idle_callback, model);
 	}
 }
 
 static void
-first_child_ref (NautilusTreeModel *model, TreeNode *node)
+stop_monitoring_directory_and_children (NautilusTreeModel *model, TreeNode *node)
 {
-	g_assert (node->all_children_ref_count == 1);
-	start_monitoring_directory (model, node);
+	TreeNode *child;
+
+	stop_monitoring_directory (model, node);
+	for (child = node->first_child; child != NULL; child = child->next) {
+		stop_monitoring_directory_and_children (model, child);
+	}
 }
 
 static void
-last_child_unref (NautilusTreeModel *model, TreeNode *node)
+stop_monitoring (NautilusTreeModel *model)
 {
-	/* Destroying nodes in here causes trouble, at least when we
-	 * use GtkTreeModelSort. I'm not sure if this is a bug or by
-	 * design. For now, sidestep this issue by deferring until idle.
-	 */
-	g_assert (node->all_children_ref_count == 0);
-	schedule_destroy_unneeded_children (model);
+	stop_monitoring_directory_and_children (model, model->details->root_node);
 }
 
 static void
@@ -1251,7 +1307,10 @@ nautilus_tree_model_ref_node (GtkTreeModel *model, GtkTreeIter *iter)
 	} else {
 		g_assert (parent->all_children_ref_count >= 0);
 		if (++parent->all_children_ref_count == 1) {
-			first_child_ref (NAUTILUS_TREE_MODEL (model), parent);
+			if (parent->first_child == NULL) {
+				parent->done_loading = FALSE;
+			}
+			schedule_monitoring_update (NAUTILUS_TREE_MODEL (model));
 		}
 #if LOG_REF_COUNTS
 		uri = get_node_uri (iter);
@@ -1295,7 +1354,7 @@ nautilus_tree_model_unref_node (GtkTreeModel *model, GtkTreeIter *iter)
 		g_free (uri);
 #endif
 		if (--parent->all_children_ref_count == 0) {
-			last_child_unref (NAUTILUS_TREE_MODEL (model), parent);
+			schedule_monitoring_update (NAUTILUS_TREE_MODEL (model));
 		}
 	}
 }
@@ -1320,9 +1379,8 @@ nautilus_tree_model_set_root_uri (NautilusTreeModel *model, const char *root_uri
 	node = create_node_for_file (model, file);
 	model->details->root_node = node;
 
-	model->details->root_node_changed_id = g_signal_connect
-		(file, "changed",
-		 G_CALLBACK (root_node_file_changed_callback), model);
+	g_signal_connect_object	(file, "changed",
+				 G_CALLBACK (root_node_file_changed_callback), model, 0);
 
 	attrs = get_tree_monitor_attributes ();
 	nautilus_file_monitor_add (file, model, attrs);
@@ -1345,6 +1403,69 @@ nautilus_tree_model_new (const char *root_uri)
 	return model;
 }
 
+void
+nautilus_tree_model_set_show_hidden_files (NautilusTreeModel *model,
+					   gboolean show_hidden_files)
+{
+	g_return_if_fail (NAUTILUS_IS_TREE_MODEL (model));
+	g_return_if_fail (show_hidden_files == FALSE || show_hidden_files == TRUE);
+
+	show_hidden_files = show_hidden_files != FALSE;
+	if (model->details->show_hidden_files == show_hidden_files) {
+		return;
+	}
+	model->details->show_hidden_files = show_hidden_files;
+	stop_monitoring (model);
+	if (!show_hidden_files) {
+		destroy_by_function (model, nautilus_file_is_hidden_file);
+	}
+	schedule_monitoring_update (model);
+}
+
+void
+nautilus_tree_model_set_show_backup_files (NautilusTreeModel *model,
+					   gboolean show_backup_files)
+{
+	g_return_if_fail (NAUTILUS_IS_TREE_MODEL (model));
+	g_return_if_fail (show_backup_files == FALSE || show_backup_files == TRUE);
+
+	show_backup_files = show_backup_files != FALSE;
+	if (model->details->show_backup_files == show_backup_files) {
+		return;
+	}
+	model->details->show_backup_files = show_backup_files;
+	stop_monitoring (model);
+	if (!show_backup_files) {
+		destroy_by_function (model, nautilus_file_is_backup_file);
+	}
+	schedule_monitoring_update (model);
+}
+
+static gboolean
+file_is_not_directory (NautilusFile *file)
+{
+	return !nautilus_file_is_directory (file);
+}
+
+void
+nautilus_tree_model_set_show_only_directories (NautilusTreeModel *model,
+					       gboolean show_only_directories)
+{
+	g_return_if_fail (NAUTILUS_IS_TREE_MODEL (model));
+	g_return_if_fail (show_only_directories == FALSE || show_only_directories == TRUE);
+
+	show_only_directories = show_only_directories != FALSE;
+	if (model->details->show_only_directories == show_only_directories) {
+		return;
+	}
+	model->details->show_only_directories = show_only_directories;
+	stop_monitoring (model);
+	if (show_only_directories) {
+		destroy_by_function (model, file_is_not_directory);
+	}
+	schedule_monitoring_update (model);
+}
+
 NautilusFile *
 nautilus_tree_model_iter_get_file (NautilusTreeModel *model, GtkTreeIter *iter)
 {
@@ -1379,18 +1500,17 @@ nautilus_tree_model_finalize (GObject *object)
 
 	root = model->details->root_node;
 	if (root != NULL) {
-		g_signal_handler_disconnect (root->file, model->details->root_node_changed_id);
 		nautilus_file_monitor_remove (root->file, model);
 		destroy_node_without_reporting (model, root);
 	}
 
-	if (model->details->destroy_unneeded_children_idle_id != 0) {
-		g_source_remove (model->details->destroy_unneeded_children_idle_id);
+	if (model->details->monitoring_update_idle_id != 0) {
+		g_source_remove (model->details->monitoring_update_idle_id);
 	}
 
 	g_free (model->details);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	parent_class->finalize (object);
 }
 
 static void
