@@ -99,7 +99,6 @@ eel_gtk_main_quit_all (void)
 static void
 event_loop_unregister (GtkObject *object)
 {
-	g_assert (g_slist_find (event_loop_registrants, object) != NULL);
 	event_loop_registrants = g_slist_remove (event_loop_registrants, object);
 	if (!is_event_loop_needed ()) {
 		eel_gtk_main_quit_all ();
@@ -192,8 +191,6 @@ main (int argc, char *argv[])
 
 	popt_context = g_value_get_pointer (&context_as_value);
 
-	eel_setenv ("DISPLAY", DisplayString (GDK_DISPLAY ()), TRUE);
-
 	/* Check for argument consistency. */
 	args = poptGetArgs (popt_context);
 	if (perform_self_check && args != NULL) {
@@ -219,8 +216,6 @@ main (int argc, char *argv[])
 
 	/* Initialize the services that we use. */
 	LIBXML_TEST_VERSION
-	gnome_vfs_init ();
-	bonobo_init (&argc, argv);
 
 	if (g_getenv ("NAUTILUS_ENABLE_TEST_COMPONENTS") != NULL) {
 		bonobo_activation_set_test_components_enabled (TRUE);
