@@ -172,6 +172,32 @@ nautilus_window_instance_init (NautilusWindow *window)
 {
 	window->details = g_new0 (NautilusWindowDetails, 1);
 
+	/* CORBA and Bonobo setup, which must be done before the location bar setup */
+	window->details->ui_container = bonobo_window_get_ui_container (BONOBO_WINDOW (window));
+	bonobo_object_ref (window->details->ui_container);
+
+	window->details->tooltips = gtk_tooltips_new ();
+	g_object_ref (G_OBJECT (window->details->tooltips));
+	gtk_object_sink (GTK_OBJECT (window->details->tooltips));
+
+	/* Set initial window title */
+	gtk_window_set_title (GTK_WINDOW (window), _("Nautilus"));
+
+	window->details->shell_ui = bonobo_ui_component_new ("Nautilus Shell");
+	bonobo_ui_component_set_container
+		(window->details->shell_ui,
+		 nautilus_window_get_ui_container (window),
+		 NULL);
+
+	/* Create a separate component so when we remove the status
+	 * we don't loose the status bar
+	 */
+      	window->details->status_ui = bonobo_ui_component_new ("Status Component");  
+	bonobo_ui_component_set_container
+		(window->details->status_ui,
+		 nautilus_window_get_ui_container (window),
+		 NULL);
+
 	gtk_quit_add_destroy (1, GTK_OBJECT (window));
 
 	/* Keep track of changes in enabled state of sidebar panels */
@@ -397,7 +423,7 @@ nautilus_window_zoom_in (NautilusWindow *window)
 }
 
 void
-nautilus_window_zoom_to_level (NautilusWindow *window, double level)
+nautilus_window_zoom_to_level (NautilusWindow *window, float level)
 {
 	if (window->content_view != NULL) {
 		nautilus_view_frame_set_zoom_level (window->content_view, level);
@@ -536,35 +562,13 @@ nautilus_window_constructed (NautilusWindow *window)
 	GtkWidget *view_as_menu_vbox;
 	BonoboControl *location_bar_wrapper;
 	
-	/* CORBA and Bonobo setup, which must be done before the location bar setup */
-	window->details->ui_container = bonobo_ui_container_new ();
-	bonobo_ui_container_set_engine (window->details->ui_container,
-					bonobo_window_get_ui_engine (BONOBO_WINDOW (window)));
-
-	/* Load the user interface from the XML file. */
-	window->details->shell_ui = bonobo_ui_component_new ("Nautilus Shell");
-	bonobo_ui_component_set_container
-		(window->details->shell_ui,
-		 nautilus_window_get_ui_container (window),
-		 NULL);
-
-	/* Create a separate component so when we remove the status
-	 * we don't loose the status bar
-	 */
-      	window->details->status_ui = bonobo_ui_component_new ("Status Component");
-	bonobo_ui_component_set_container
-		(window->details->status_ui,
-		 nautilus_window_get_ui_container (window),
-		 NULL);
-
 	nautilus_window_ui_freeze (window);
 
-	bonobo_ui_component_freeze (window->details->shell_ui, NULL);
+	/* Load the user interface from the XML file. */
 	bonobo_ui_util_set_ui (window->details->shell_ui,
 			       DATADIR,
 			       "nautilus-shell-ui.xml",
 			       "nautilus", NULL);
-	bonobo_ui_component_thaw (window->details->shell_ui, NULL);
 	
 	/* set up location bar */
 	location_bar_box = gtk_hbox_new (FALSE, GNOME_PAD);
@@ -784,6 +788,11 @@ nautilus_window_destroy (GtkObject *object)
 		window->content_view = NULL;
 	}
 
+	if (window->details->tooltips) {
+		g_object_unref (G_OBJECT (window->details->tooltips));
+		window->details->tooltips = NULL;
+	}
+
 	GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
